@@ -22,44 +22,44 @@ from Bio.codonalign.codonalphabet import default_codon_table, default_codon_alph
 from Bio.codonalign.codonalphabet import get_codon_alphabet as _get_codon_alphabet
 
 import warnings
-warnings.warn('Bio.codonalign is an experimental module which may undergo '
-              'significant changes prior to its future official release.',
+warnings.warn("Bio.codonalign is an experimental module which may undergo "
+              "significant changes prior to its future official release.",
               BiopythonExperimentalWarning)
 
 
-def build(pro_align, nucl_seqs, corr_dict=None, gap_char='-', unknown='X',
+def build(pro_align, nucl_seqs, corr_dict=None, gap_char="-", unknown="X",
           codon_table=default_codon_table, alphabet=None,
           complete_protein=False, anchor_len=10, max_score=10):
     """Build a codon alignment from protein alignment and corresponding nucleotides.
 
     Arguments:
      - pro_align  - a protein MultipleSeqAlignment object
-     - nucl_align - an object returned by SeqIO.parse or SeqIO.index
+     - nucl_seqs - an object returned by SeqIO.parse or SeqIO.index
        or a collection of SeqRecord.
      - alphabet   - alphabet for the returned codon alignment
      - corr_dict  - a dict that maps protein id to nucleotide id
      - complete_protein - whether the sequence begins with a start
        codon
-     - frameshift - whether to apply frameshift detection
 
-    Return a CodonAlignment object
+    Return a CodonAlignment object.
 
-    >>> from Bio.Alphabet import IUPAC
+    The example below answers this Biostars question: https://www.biostars.org/p/89741/
+
+    >>> from Bio.Alphabet import generic_dna, generic_protein
     >>> from Bio.Seq import Seq
     >>> from Bio.SeqRecord import SeqRecord
     >>> from Bio.Align import MultipleSeqAlignment
-    >>> seq1 = SeqRecord(Seq('TCAGGGACTGCGAGAACCAAGCTACTGCTGCTGCTGGCTGCGCTCTGCGCCGCAGGTGGGGCGCTGGAG',
-    ...     alphabet=IUPAC.IUPACUnambiguousDNA()), id='pro1')
-    >>> seq2 = SeqRecord(Seq('TCAGGGACTTCGAGAACCAAGCGCTCCTGCTGCTGGCTGCGCTCGGCGCCGCAGGTGGAGCACTGGAG',
-    ...     alphabet=IUPAC.IUPACUnambiguousDNA()), id='pro2')
-    >>> pro1 = SeqRecord(Seq('SGTARTKLLLLLAALCAAGGALE', alphabet=IUPAC.protein),id='pro1')
-    >>> pro2 = SeqRecord(Seq('SGTSRTKRLLLLAALGAAGGALE', alphabet=IUPAC.protein),id='pro2')
+    >>> from Bio.codonalign import build
+    >>> seq1 = SeqRecord(Seq('ATGTCTCGT', alphabet=generic_dna), id='pro1')
+    >>> seq2 = SeqRecord(Seq('ATGCGT', alphabet=generic_dna), id='pro2')
+    >>> pro1 = SeqRecord(Seq('MSR', alphabet=generic_protein), id='pro1')
+    >>> pro2 = SeqRecord(Seq('M-R', alphabet=generic_protein), id='pro2')
     >>> aln = MultipleSeqAlignment([pro1, pro2])
     >>> codon_aln = build(aln, [seq1, seq2])
     >>> print(codon_aln)
-    CodonAlphabet(Standard) CodonAlignment with 2 rows and 69 columns (23 codons)
-    TCAGGGACTGCGAGAACCAAGCTACTGCTGCTGCTGGCTGCGCTCTGCGCCGCAGGT...GAG pro1
-    TCAGGGACTTCGAGAACCAAGCG-CTCCTGCTGCTGGCTGCGCTCGGCGCCGCAGGT...GAG pro2
+    CodonAlphabet(Standard) CodonAlignment with 2 rows and 9 columns (3 codons)
+    ATGTCTCGT pro1
+    ATG---CGT pro2
 
     """
     # TODO
@@ -115,8 +115,7 @@ def build(pro_align, nucl_seqs, corr_dict=None, gap_char='-', unknown='X',
                 from Bio import SeqIO
                 nucl_seqs = SeqIO.to_dict(nucl_seqs)
             elif nucl_seqs.__class__.__name__ in ("list", "tuple"):
-                nucl_seqs = dict((i.id, i) for i in nucl_seqs)
-                # nucl_seqs = {i.id: i for i in nucl_seqs}
+                nucl_seqs = {i.id: i for i in nucl_seqs}
             elif nucl_seqs.__class__.__name__ in \
                     ("_IndexedSeqFileDict", "dict"):
                 pass
@@ -136,13 +135,13 @@ def build(pro_align, nucl_seqs, corr_dict=None, gap_char='-', unknown='X',
     # corr_method = 1, keyword pairing
     elif corr_method == 1:
         nucl_id = set(nucl_seqs.keys())
-        pro_id = set(i.id for i in pro_align)
+        pro_id = {i.id for i in pro_align}
         # check if there is pro_id that does not have a nucleotide match
         if pro_id - nucl_id:
             diff = pro_id - nucl_id
             raise ValueError("Protein Record {0} cannot find a nucleotide "
                              "sequence match, please check the "
-                             "id".format(', '.join(diff)))
+                             "id".format(", ".join(diff)))
         else:
             pro_nucl_pair = []
             for pro_rec in pro_align:
@@ -187,17 +186,17 @@ def build(pro_align, nucl_seqs, corr_dict=None, gap_char='-', unknown='X',
 
 def _codons2re(codons):
     """Generate regular expression based on a given list of codons (PRIVATE)."""
-    reg = ''
+    reg = ""
     for i in zip(*codons):
         if len(set(i)) == 1:
-            reg += ''.join(set(i))
+            reg += "".join(set(i))
         else:
-            reg += '[' + ''.join(set(i)) + ']'
+            reg += "[" + "".join(set(i)) + "]"
     return reg
 
 
-def _get_aa_regex(codon_table, stop='*', unknown='X'):
-    """Set up the regular expression of a given CodonTable.
+def _get_aa_regex(codon_table, stop="*", unknown="X"):
+    """Set up the regular expression of a given CodonTable (PRIVATE).
 
     >>> from Bio.Data.CodonTable import generic_by_id
     >>> p = generic_by_id[1]
@@ -226,13 +225,13 @@ def _get_aa_regex(codon_table, stop='*', unknown='X'):
     for aa, codons in aa2codon.items():
         aa2codon[aa] = _codons2re(codons)
     aa2codon[stop] = _codons2re(codon_table.stop_codons)
-    aa2codon[unknown] = '...'
+    aa2codon[unknown] = "..."
     return aa2codon
 
 
-def _check_corr(pro, nucl, gap_char='-', codon_table=default_codon_table,
+def _check_corr(pro, nucl, gap_char="-", codon_table=default_codon_table,
                 complete_protein=False, anchor_len=10):
-    """Check if the nucleotide can be translated into the protein.
+    """Check if the nucleotide can be translated into the protein (PRIVATE).
 
     Expects two SeqRecord objects.
     """
@@ -244,7 +243,7 @@ def _check_corr(pro, nucl, gap_char='-', codon_table=default_codon_table,
                         "check your input.")
 
     def get_alpha(alpha):
-        if hasattr(alpha, 'alphabet'):
+        if hasattr(alpha, "alphabet"):
             return get_alpha(alpha.alphabet)
         else:
             return alpha
@@ -296,17 +295,17 @@ def _check_corr(pro, nucl, gap_char='-', codon_table=default_codon_table,
                 for aa in anchor:
                     if complete_protein and i == 0:
                         qcodon += _codons2re(codon_table.start_codons)
-                        fncodon += aa2re['X']
+                        fncodon += aa2re["X"]
                         continue
                     qcodon += aa2re[aa]
-                    fncodon += aa2re['X']
+                    fncodon += aa2re["X"]
                 match = re.search(qcodon, nucl_seq)
             elif this_anchor_len > anchor_len:
                 last_qcodon = ""
                 last_fcodon = ""
                 for j in range(anchor_len, len(anchor)):
                     last_qcodon += aa2re[anchor[j]]
-                    last_fcodon += aa2re['X']
+                    last_fcodon += aa2re["X"]
                 match = re.search(last_qcodon, nucl_seq)
             # build full_pro_re from anchors
             if match:
@@ -423,7 +422,7 @@ def _check_corr(pro, nucl, gap_char='-', codon_table=default_codon_table,
 
 def _get_shift_anchor_re(sh_anc, sh_nuc, shift_val, aa2re, anchor_len,
                          shift_id_pos):
-    """Find a regular expression matching a potentially shifted anchor.
+    """Find a regular expression matching a potentially shifted anchor (PRIVATE).
 
     Arguments:
      - sh_anc    - shifted anchor sequence
@@ -451,7 +450,7 @@ def _get_shift_anchor_re(sh_anc, sh_nuc, shift_val, aa2re, anchor_len,
             qcodon += "$"
             match = re.search(qcodon, sh_nuc)
             if match:
-                qcodon = qcodon.replace('^', '').replace('$', '')
+                qcodon = qcodon.replace("^", "").replace("$", "")
                 shift_id_pos += 1
                 return qcodon, shift_id_pos
         if not match:
@@ -474,10 +473,10 @@ def _get_shift_anchor_re(sh_anc, sh_nuc, shift_val, aa2re, anchor_len,
                             shift_id[shift_id_pos].upper())
                 else:
                     qcodon += aa2re[aa]
-            qcodon += '$'
+            qcodon += "$"
             match = re.search(qcodon, sh_nuc)
             if match:
-                qcodon = qcodon.replace('^', '').replace('$', '')
+                qcodon = qcodon.replace("^", "").replace("$", "")
                 shift_id_pos += 1
                 return qcodon, shift_id_pos
         if not match:
@@ -491,10 +490,10 @@ def _merge_aa2re(aa1, aa2, shift_val, aa2re, reid):
         aas = []
         m = 0
         for i in re_aa:
-            if i == '[':
+            if i == "[":
                 m = -1
-                aas.append('')
-            elif i == ']':
+                aas.append("")
+            elif i == "]":
                 m = 0
                 continue
             elif m == -1:
@@ -504,22 +503,22 @@ def _merge_aa2re(aa1, aa2, shift_val, aa2re, reid):
         return aas
     scodon = list(map(get_aa_from_codonre, (aa2re[aa1], aa2re[aa2])))
     if shift_val == 1:
-        intersect = ''.join(set(scodon[0][2]) & set(scodon[1][0]))
-        scodonre = '(?P<' + reid + '>'
-        scodonre += '[' + scodon[0][0] + ']' + \
-                    '[' + scodon[0][1] + ']' + \
-                    '[' + intersect + ']' + \
-                    '[' + scodon[1][1] + ']' + \
-                    '[' + scodon[1][2] + ']'
+        intersect = "".join(set(scodon[0][2]) & set(scodon[1][0]))
+        scodonre = "(?P<" + reid + ">"
+        scodonre += "[" + scodon[0][0] + "]" + \
+                    "[" + scodon[0][1] + "]" + \
+                    "[" + intersect + "]" + \
+                    "[" + scodon[1][1] + "]" + \
+                    "[" + scodon[1][2] + "]"
     elif shift_val == 2:
-        intersect1 = ''.join(set(scodon[0][1]) & set(scodon[1][0]))
-        intersect2 = ''.join(set(scodon[0][2]) & set(scodon[1][1]))
-        scodonre = '(?P<' + reid + '>'
-        scodonre += '[' + scodon[0][0] + ']' + \
-                    '[' + intersect1 + ']' + \
-                    '[' + intersect2 + ']' + \
-                    '[' + scodon[1][2] + ']'
-    scodonre += ')'
+        intersect1 = "".join(set(scodon[0][1]) & set(scodon[1][0]))
+        intersect2 = "".join(set(scodon[0][2]) & set(scodon[1][1]))
+        scodonre = "(?P<" + reid + ">"
+        scodonre += "[" + scodon[0][0] + "]" + \
+                    "[" + intersect1 + "]" + \
+                    "[" + intersect2 + "]" + \
+                    "[" + scodon[1][2] + "]"
+    scodonre += ")"
     return scodonre
 
 
@@ -604,7 +603,7 @@ def _get_codon_rec(pro, nucl, span_mode, alphabet, gap_char="-",
                     m_groupdict[shift_start.index(i)].isupper():
                 shift_index = shift_start.index(i)
                 shift_val = 6 - (shift_pos[shift_index][1] -
-                            shift_pos[shift_index][0])
+                                 shift_pos[shift_index][0])
                 rf_table.append(i)
                 rf_table.append(i + 3 - shift_val)
                 i = shift_pos[shift_index][1]
@@ -629,20 +628,21 @@ def _get_codon_rec(pro, nucl, span_mode, alphabet, gap_char="-",
                     codon_seq += this_codon
                     aa_num += 1
             else:
-                if aa_num < len(pro.seq.ungap('-')) - 1 and \
+                if aa_num < len(pro.seq.ungap("-")) - 1 and \
                         rf_table[aa_num + 1] - rf_table[aa_num] - 3 < 0:
                     max_score -= 1
                     start = rf_table[aa_num]
                     end = start + (3 - shift_val)
                     ngap = shift_val
-                    this_codon = nucl_seq._data[start:end] + '-' * ngap
+                    this_codon = nucl_seq._data[start:end] + "-" * ngap
                 elif rf_table[aa_num] - rf_table[aa_num - 1] - 3 > 0:
                     max_score -= 1
                     start = rf_table[aa_num - 1] + 3
                     end = rf_table[aa_num]
                     ngap = 3 - (rf_table[aa_num] - rf_table[aa_num - 1] - 3)
-                    this_codon = nucl_seq._data[start:end] + '-' * ngap + \
-                            nucl_seq._data[rf_table[aa_num]:rf_table[aa_num] + 3]
+                    this_codon = (nucl_seq._data[start:end]
+                                  + "-" * ngap
+                                  + nucl_seq._data[rf_table[aa_num]:rf_table[aa_num] + 3])
                 else:
                     start = rf_table[aa_num]
                     end = start + 3
@@ -709,13 +709,13 @@ def _align_shift_recs(recs):
                 else:
                     gap_num = 0
                 if gap_num != 0:
-                    gaps = '-' * int(gap_num)
-                    seq = recs[j].seq._data[:int(k)] + gaps + \
-                            recs[j].seq._data[int(k):]
+                    gaps = "-" * int(gap_num)
+                    seq = (recs[j].seq._data[:int(k)] + gaps
+                           + recs[j].seq._data[int(k):])
                     full_rf_table = full_rf_table_lst[j]
                     bp = full_rf_table.index(k)
-                    full_rf_table = full_rf_table[:bp] + \
-                            [v + int(gap_num) for v in full_rf_table[bp + 1:]]
+                    full_rf_table = (full_rf_table[:bp]
+                                     + [v + int(gap_num) for v in full_rf_table[bp + 1:]])
                     full_rf_table_lst[j] = full_rf_table
                     recs[j].seq = CodonSeq(seq,
                                            rf_table=recs[j].seq.rf_table,
@@ -726,15 +726,15 @@ def _align_shift_recs(recs):
         if len(add_lst) != rec_num:
             for j, k in add_lst:
                 gaps = "-" * int(gap_num)
-                seq = recs[j].seq._data[:int(k)] + gaps + \
-                        recs[j].seq._data[int(k):]
+                seq = (recs[j].seq._data[:int(k)] + gaps
+                       + recs[j].seq._data[int(k):])
                 full_rf_table = full_rf_table_lst[j]
                 bp = full_rf_table.index(k)
                 inter_rf = []
                 for t in filter(lambda x: x % 3 == 0, range(len(gaps))):
                     inter_rf.append(k + t + 3.0)
-                full_rf_table = full_rf_table[:bp] + inter_rf + \
-                        [v + int(gap_num) for v in full_rf_table[bp:]]
+                full_rf_table = (full_rf_table[:bp] + inter_rf
+                                 + [v + int(gap_num) for v in full_rf_table[bp:]])
                 full_rf_table_lst[j] = full_rf_table
                 recs[j].seq = CodonSeq(seq,
                                        rf_table=recs[j].seq.rf_table,
